local mod = ...



-- *************************************************************************************************************
-- Custom Component: Outlaw Theft Point -------------------------------------------------------------------------------------
-- *************************************************************************************************************
local COMP_SECURITY_OUTLAWTHEFTPOINT = {
	TypeName = "COMP_SECURITY_OUTLAWTHEFTPOINT",
	ParentType = "COMPONENT",
	Properties = {
		{ Name = "DayCreated", Type = "integer", Default = -1, Flags = { "SAVE_GAME" } },
		{ Name = "Reserved", Type = "boolean", Default = false, Flags = { "SAVE_GAME" } },
		{ Name = "Resource", Type = "string", Default = "GOLD_COINS", Flags = { "SAVE_GAME" } },
    }
}


-- Init -------------------------------------------------------------------------------------
function COMP_SECURITY_OUTLAWTHEFTPOINT:init()

    -- mod:log("Theft point day created: " .. tostring(self.DayCreated))


    -- Daily script
    local compMainGameLoop = self:getLevel():find("COMP_MAIN_GAME_LOOP")
    event.register(self, compMainGameLoop.ON_NEW_DAY, function()

        local currentDay = CurrentDay(self)
        if (self.DayCreated == -1) then
            self.DayCreated = currentDay
        end
    
        -- mod:log("Theft point day created: " .. tostring(self.DayCreated) .. " vs current day: " .. tostring(currentDay))

        -- Delete if expired (after 3 days), but if it is reserved for use leave it
        if (self.DayCreated < (currentDay - 2)) and not (self.Reserved) then
            self:getOwner():destroy()
        end

    end)
        
end


-- Register custom Class
mod:registerClass(COMP_SECURITY_OUTLAWTHEFTPOINT)



-- *************************************************************************************************************
-- Custom Component: Security Building Part -------------------------------------------------------------------------------------
-- Stores a building part's security score
-- *************************************************************************************************************
local COMP_SECURITY_BUILDING_PART = {
	TypeName = "COMP_SECURITY_BUILDING_PART",
	ParentType = "COMPONENT",
	Properties = {
		{ Name = "Score", Type = "integer", Default = 0, Flags = { "SAVE_GAME" } },
		{ Name = "DayScoreChanged", Type = "integer", Default = 0, Flags = { "SAVE_GAME" } },
		{ Name = "DaySpawnedTheftPoint", Type = "integer", Default = 0, Flags = { "SAVE_GAME" } },
    }
}

-- Register custom Class
mod:registerClass(COMP_SECURITY_BUILDING_PART)



-- -- *************************************************************************************************************
-- -- Custom Component: Prisoner Position -------------------------------------------------------------------------------------
-- -- Prisoners go to these positions during their work cycle
-- -- *************************************************************************************************************
-- local COMP_SECURITY_PRISONERPOSITION = {
-- 	TypeName = "COMP_SECURITY_PRISONERPOSITION",
-- 	ParentType = "COMPONENT",
-- 	Properties = {
-- 		{ Name = "Reserved", Type = "boolean", Default = false, Flags = { "SAVE_GAME" } },
-- 		{ Name = "DayReserved", Type = "integer", Default = -1, Flags = { "SAVE_GAME" } },
--     }
-- }


-- -- Init -------------------------------------------------------------------------------------
-- function COMP_SECURITY_PRISONERPOSITION:init()

--     -- Daily script
--     local compMainGameLoop = self:getLevel():find("COMP_MAIN_GAME_LOOP")
--     event.register(self, compMainGameLoop.ON_NEW_DAY, function()

--         local currentDay = CurrentDay(self)
--         if (self.DayReserved < (currentDay - 5)) then -- it's been too long since reservation, free up this position (outlaw got lost?)
--             self.Reserved = false
--             self.DayReserved = -1
--         end
    
--     end)
        
-- end


-- -- Register custom Class
-- mod:registerClass(COMP_SECURITY_PRISONERPOSITION)



-- *************************************************************************************************************
-- Custom Component: Security Villager -------------------------------------------------------------------------------------
-- Stores the length a villager has been unemployed
-- *************************************************************************************************************
local COMP_SECURITY_VILLAGER = {
	TypeName = "COMP_SECURITY_VILLAGER",
	ParentType = "COMPONENT",
	Properties = {
		{ Name = "DayUnemployed", Type = "integer", Default = 0, Flags = { "SAVE_GAME" } },
		{ Name = "DefenderFlag", Type = "boolean", Default = false, Flags = { "SAVE_GAME" } },
		{ Name = "OutlawFlag", Type = "boolean", Default = false, Flags = { "SAVE_GAME" } },
		{ Name = "PrisonerFlag", Type = "boolean", Default = false, Flags = { "SAVE_GAME" } },
		{ Name = "Questioning", Type = "boolean", Default = false, Flags = { "SAVE_GAME" } },
		{ Name = "QuestioningVillager", Type = "COMP_SECURITY_VILLAGER", Default = nil, Flags = { "SAVE_GAME" } },
		{ Name = "TheftPoint", Type = "COMP_SECURITY_OUTLAWTHEFTPOINT", Default = nil, Flags = { "SAVE_GAME" } },
		{ Name = "OutlawPosition", Type = "COMP_SECURITY_OUTLAWPOSITION", Default = nil, Flags = { "SAVE_GAME" } },
		{ Name = "RallyPosition", Type = "COMP_SECURITY_RALLYPOSITION", Default = nil, Flags = { "SAVE_GAME" } },
		{ Name = "SoldierPosition", Type = "COMP_SECURITY_SOLDIERPOSITION", Default = nil, Flags = { "SAVE_GAME" } },
		-- { Name = "PrisonerPosition", Type = "COMP_SECURITY_PRISONERPOSITION", Default = nil, Flags = { "SAVE_GAME" } },
    }
}

-- Register custom Class
mod:registerClass(COMP_SECURITY_VILLAGER)



-- *************************************************************************************************************
-- Custom Component: Patrol Arrow -------------------------------------------------------------------------------------
-- Keeps track of whether it has been used in the last n seconds
-- *************************************************************************************************************
local COMP_SECURITY_GUARD_ARROW = {
	TypeName = "COMP_SECURITY_GUARD_ARROW",
	ParentType = "COMPONENT",
	Properties = {
		{ Name = "Used", Type = "boolean", Default = false, Flags = { "SAVE_GAME" } },
		{ Name = "DayUsed", Type = "integer", Default = -1, Flags = { "SAVE_GAME" } },
    }
}


-- Init -------------------------------------------------------------------------------------
function COMP_SECURITY_GUARD_ARROW:init()

    -- Daily script
    local compMainGameLoop = self:getLevel():find("COMP_MAIN_GAME_LOOP")
    event.register(self, compMainGameLoop.ON_NEW_DAY, function()

        local currentDay = CurrentDay(self)
        if (self.DayUsed > 0) and (self.DayUsed < (currentDay - 3)) then -- expired after four days
            self.Used = false
            self.DayUsed = -1
        end

    end)

end


-- Register custom Class
mod:registerClass(COMP_SECURITY_GUARD_ARROW)

-- Attach class to prefab
mod:registerPrefabComponent("PREFAB_SECURITY_PATROLPOINT", { DataType = "COMP_SECURITY_GUARD_ARROW" } )


